// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:example_router/core/utils/custom_password.dart';
import 'package:flutter/material.dart';
import 'package:wyatt_authentication_bloc/wyatt_authentication_bloc.dart';
import 'package:wyatt_form_bloc/wyatt_form_bloc.dart';

class _EmailInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return InputBuilder<EditAccountCubit<int>>(
      field: AuthFormField.email,
      builder: ((context, cubit, state, field, input) {
        return TextField(
          onChanged: (email) => cubit.emailChanged(email),
          keyboardType: TextInputType.emailAddress,
          decoration: InputDecoration(
            labelText: 'Email',
            helperText: '',
            errorText: input.validator.invalid ? 'Invalid email' : null,
          ),
        );
      }),
    );
  }
}

class _PasswordInput extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return InputBuilder<EditAccountCubit<int>>(
      field: AuthFormField.password,
      builder: ((context, cubit, state, field, input) {
        return TextField(
          onChanged: (pwd) => cubit
              .passwordCustomChanged<CustomPassword>(CustomPassword.dirty(pwd)),
          obscureText: true,
          decoration: InputDecoration(
            labelText: 'Password',
            helperText: '',
            errorText: input.validator.invalid ? 'Invalid password' : null,
          ),
        );
      }),
    );
  }
}

class _EditAccountButton extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return SubmitBuilder<EditAccountCubit<int>>(
      builder: ((context, cubit, status) {
        return status.isSubmissionInProgress
            ? const CircularProgressIndicator()
            : ElevatedButton(
                onPressed: status.isValidated
                    ? () {
                        cubit.updateEmail();
                        cubit.updatePassword();
                      }
                    : null,
                child: const Text('Update Account'),
              );
      }),
    );
  }
}

class _EditAccountEmailButton extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return SubmitBuilder<EditAccountCubit<int>>(
      builder: ((context, cubit, status) {
        return status.isSubmissionInProgress
            ? const CircularProgressIndicator()
            : ElevatedButton(
                onPressed: status.isValidated
                    ? () {
                        cubit.updateEmail();
                      }
                    : null,
                child: const Text('Update Account Email'),
              );
      }),
    );
  }
}

class _EditAccountPasswordButton extends StatelessWidget {
  @override
  Widget build(BuildContext context) {
    return SubmitBuilder<EditAccountCubit<int>>(
      builder: ((context, cubit, status) {
        return status.isSubmissionInProgress
            ? const CircularProgressIndicator()
            : ElevatedButton(
                onPressed: status.isValidated
                    ? () {
                        cubit.updatePassword();
                      }
                    : null,
                child: const Text('Update Account Password'),
              );
      }),
    );
  }
}

class EditAccountForm extends StatelessWidget {
  const EditAccountForm({Key? key}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return EditAccountListener<int>(
      onError: (context, status, errorMessage) => ScaffoldMessenger.of(context)
        ..hideCurrentSnackBar()
        ..showSnackBar(
          SnackBar(content: Text(errorMessage ?? 'Account edition Failure')),
        ),
      child: ListView(
        shrinkWrap: true,
        children: [
          _EmailInput(),
          const SizedBox(height: 8),
          _PasswordInput(),
          const SizedBox(height: 16),
          _EditAccountButton(),
          const SizedBox(height: 16),
          _EditAccountEmailButton(),
          const SizedBox(height: 16),
          _EditAccountPasswordButton(),
        ],
      ),
    );
  }
}
