// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_bloc_helper/wyatt_bloc_helper.dart';
import 'package:wyatt_cloud_messaging_bloc_base/src/src.dart';

class CloudMessagingWrapperScreen
    extends CubitScreen<CloudMessagingCubit, CloudmessagingState> {
  const CloudMessagingWrapperScreen({
    required this.child,
    required this.onNotification,
    required this.listenNotificationUseCase,
    this.subscribeToNotificationTopicUseCase,
    this.unsubscribeFromNotificationTopicUseCase,
    super.key,
  });

  final ListenNotificationUseCase listenNotificationUseCase;
  final SubscribeToNotificationTopicUseCase?
      subscribeToNotificationTopicUseCase;
  final UnsubscribeFromNotificationTopicUseCase?
      unsubscribeFromNotificationTopicUseCase;

  final OnNotification onNotification;

  final Widget child;

  @override
  CloudMessagingCubit create(BuildContext context) => CloudMessagingCubit(
        listenNotification: listenNotificationUseCase,
        subscribe: subscribeToNotificationTopicUseCase,
        unsubscribe: unsubscribeFromNotificationTopicUseCase,
      )..listenNotification();

  @override
  void onListen(BuildContext context, CloudmessagingState state) {
    if (state.remoteNotification != null) {
      onNotification.call(state.remoteNotification!);
    }
  }

  @override
  Widget onBuild(BuildContext context, CloudmessagingState state) => child;
}
