// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'dart:convert';

import 'package:wyatt_i18n/src/core/exceptions/exceptions.dart';
import 'package:wyatt_i18n/src/core/utils/parser.dart';

/// {@template json_parser}
/// A class that parses a given input of type [String] into a given output
/// of type [Map<String, dynamic>].
/// {@endtemplate}
class JsonParser extends Parser<String, Map<String, dynamic>> {
  /// {@macro json_parser}
  const JsonParser() : super();

  /// Parses the given [input] of type [String] into a given output of type
  /// [Map<String, dynamic>].
  @override
  Map<String, dynamic> parse(String input) {
    try {
      return jsonDecode(input) as Map<String, dynamic>;
    } catch (e, s) {
      throw ParserException(e.toString(), s);
    }
  }
}
