// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';

class TopBarThemeExtension extends ThemeExtension<TopBarThemeExtension> {
  const TopBarThemeExtension({
    this.iconTheme,
    this.backgroundColors,
    this.titleStyle,
    this.subtitleStyle,
    this.selectedIndicatorColors,
    this.selectedIndicatorSize,
    this.dividerColor,
  });

  /// Background colors from start to end of the top bar.
  final MultiColor? backgroundColors;

  /// Icon theme used in this top bar.
  final IconThemeData? iconTheme;

  /// Selected indicator colors used in this top bar.
  final MultiColor? selectedIndicatorColors;

  /// Divider color used in this top bar.
  final Color? dividerColor;

  /// Title style used in this top bar.
  final TextStyle? titleStyle;

  /// Subtitle style used in this top bar.
  final TextStyle? subtitleStyle;

  /// Selected indicator size used in this top bar.
  final Size? selectedIndicatorSize;

  @override
  ThemeExtension<TopBarThemeExtension> copyWith({
    MultiColor? backgroundColors,
    IconThemeData? iconTheme,
    MultiColor? selectedIndicatorColors,
    Color? dividerColor,
    TextStyle? titleStyle,
    TextStyle? subtitleStyle,
    Size? selectedIndicatorSize,
  }) =>
      TopBarThemeExtension(
        iconTheme: iconTheme ?? this.iconTheme,
        backgroundColors: backgroundColors ?? this.backgroundColors,
        selectedIndicatorColors:
            selectedIndicatorColors ?? this.selectedIndicatorColors,
        dividerColor: dividerColor ?? this.dividerColor,
        titleStyle: titleStyle ?? this.titleStyle,
        subtitleStyle: subtitleStyle ?? this.subtitleStyle,
        selectedIndicatorSize:
            selectedIndicatorSize ?? this.selectedIndicatorSize,
      );

  @override
  ThemeExtension<TopBarThemeExtension> lerp(
    covariant ThemeExtension<TopBarThemeExtension>? other,
    double t,
  ) {
    if (other is! TopBarThemeExtension) {
      return this;
    }
    return TopBarThemeExtension(
      iconTheme: IconThemeData.lerp(iconTheme, other.iconTheme, t),
      backgroundColors:
          MultiColor.lerp(backgroundColors, other.backgroundColors, t),
      selectedIndicatorColors: MultiColor.lerp(
        selectedIndicatorColors,
        other.selectedIndicatorColors,
        t,
      ),
      dividerColor: Color.lerp(dividerColor, other.dividerColor, t),
      titleStyle: TextStyle.lerp(titleStyle, other.titleStyle, t),
      subtitleStyle: TextStyle.lerp(subtitleStyle, other.subtitleStyle, t),
      selectedIndicatorSize:
          Size.lerp(selectedIndicatorSize, other.selectedIndicatorSize, t),
    );
  }
}
